<?php
/************************************************************************
 * This file is part of EspoCRM.
 *
 * EspoCRM - Open Source CRM application.
 * Copyright (C) 2014-2023 Yurii Kuznietsov, Taras Machyshyn, Oleksii Avramenko
 * Website: https://www.espocrm.com
 *
 * EspoCRM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EspoCRM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EspoCRM. If not, see http://www.gnu.org/licenses/.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU General Public License version 3,
 * these Appropriate Legal Notices must retain the display of the "EspoCRM" word.
 ************************************************************************/

namespace Espo\Controllers;

use Espo\Services\Integration as Service;

use Espo\Core\Exceptions\Forbidden;
use Espo\Core\Api\Request;

use Espo\Entities\User;

use stdClass;

class Integration
{
    private $service;

    private $user;

    public function __construct(Service $service, User $user)
    {
        $this->service = $service;
        $this->user = $user;

        if (!$this->user->isAdmin()) {
            throw new Forbidden();
        }
    }

    public function getActionRead(Request $request): stdClass
    {
        /** @var string $id */
        $id = $request->getRouteParam('id');

        $entity = $this->service->read($id);

        return $entity->getValueMap();
    }

    public function putActionUpdate(Request $request): stdClass
    {
        /** @var string $id */
        $id = $request->getRouteParam('id');
        $data = $request->getParsedBody();

        $entity = $this->service->update($id, $data);

        return $entity->getValueMap();
    }
}
