/************************************************************************
 * This file is part of EspoCRM.
 *
 * EspoCRM - Open Source CRM application.
 * Copyright (C) 2014-2023 Yurii Kuznietsov, Taras Machyshyn, Oleksii Avramenko
 * Website: https://www.espocrm.com
 *
 * EspoCRM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EspoCRM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EspoCRM. If not, see http://www.gnu.org/licenses/.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU General Public License version 3,
 * these Appropriate Legal Notices must retain the display of the "EspoCRM" word.
 ************************************************************************/

define('views/stream/notes/update', ['views/stream/note'], function (Dep) {

    return Dep.extend({

        template: 'stream/notes/update',

        messageName: 'update',

        data: function () {
            return _.extend({
                fieldsArr: this.fieldsArr,
                parentType: this.model.get('parentType')
            }, Dep.prototype.data.call(this));
        },

        events: {
            'click a[data-action="expandDetails"]': function (e) {
                if (this.$el.find('.details').hasClass('hidden')) {
                    this.$el.find('.details').removeClass('hidden');
                    $(e.currentTarget).find('span').removeClass('fa-chevron-down').addClass('fa-chevron-up');
                } else {
                    this.$el.find('.details').addClass('hidden');
                    $(e.currentTarget).find('span').addClass('fa-chevron-down').removeClass('fa-chevron-up');
                }
            }
        },

        init: function () {
            if (this.getUser().isAdmin()) {
                this.isRemovable = true;
            }

            Dep.prototype.init.call(this);
        },

        setup: function () {
            var data = this.model.get('data');

            var fields = data.fields;

            this.createMessage();

            this.wait(true);

            this.getModelFactory().create(this.model.get('parentType'), (model) => {
                let modelWas = model;
                let modelBecame = model.clone();

                data.attributes = data.attributes || {};

                modelWas.set(data.attributes.was);
                modelBecame.set(data.attributes.became);

                this.fieldsArr = [];

                fields.forEach(field => {
                    let type = model.getFieldType(field) || 'base';
                    let viewName = this.getMetadata()
                        .get(['entityDefs', model.name, 'fields', field, 'view']) ||
                        this.getFieldManager().getViewName(type);

                    let attributeList = this.getFieldManager().getEntityTypeFieldAttributeList(model.name, field);

                    let hasValue = false;

                    for (let attribute of attributeList) {
                        if (attribute in data.attributes.was) {
                            hasValue = true;

                            break;
                        }
                    }

                    if (!hasValue) {
                        this.fieldsArr.push({
                            field: field,
                            noValues: true,
                        });

                        return;
                    }

                    this.createView(field + 'Was', viewName, {
                        model: modelWas,
                        readOnly: true,
                        defs: {
                            name: field
                        },
                        mode: 'detail',
                        inlineEditDisabled: true,
                    });

                    this.createView(field + 'Became', viewName, {
                        model: modelBecame,
                        readOnly: true,
                        defs: {
                            name: field,
                        },
                        mode: 'detail',
                        inlineEditDisabled: true,
                    });

                    this.fieldsArr.push({
                        field: field,
                        was: field + 'Was',
                        became: field + 'Became',
                    });
                });

                this.wait(false);
            });
        },
    });
});

